@@ -30,6 +30,7 @@ int usage(const char *cmd)
         "upgrade {basedir}                    Upgrade to a new version\n"
         "adduser {basedir} [{uid}]            Adds a new user\n"
         "deluser {basedir} {uid}              Deletes a user\n"
+        "update {basedir} {uid}               Sends a user's updated profile\n"
         "httpd {basedir}                      Starts the HTTPD daemon\n"
         "purge {basedir}                      Purges old data\n"
         "state {basedir}                      Prints server state\n"
@@ -354,7 +355,7 @@ int main(int argc, char *argv[])
         xs *lid = list_maint(&snac, url, 4);
 
         if (lid != NULL) {
-            xs *lcont = list_content(&snac, lid, NULL, 0);
+            xs *lcont = list_members(&snac, lid, NULL, 0);
             const char *md5;
 
             xs_list_foreach(lcont, md5) {
@@ -410,7 +411,7 @@ int main(int argc, char *argv[])
                 if (valid_status(webfinger_request(account, &actor, &uid))) {
                     xs *md5 = xs_md5_hex(actor, strlen(actor));
 
-                    list_content(&snac, lid, md5, 1);
+                    list_members(&snac, lid, md5, 1);
                     printf("Actor %s (%s) added to list '%s' (%s)\n", actor, uid, url, lid);
                 }
                 else
@@ -433,7 +434,7 @@ int main(int argc, char *argv[])
             if (lid != NULL) {
                 xs *md5 = xs_md5_hex(account, strlen(account));
 
-                list_content(&snac, lid, md5, 2);
+                list_members(&snac, lid, md5, 2);
                 printf("Actor %s deleted from list '%s' (%s)\n", account, url, lid);
             }
             else
@@ -738,6 +739,11 @@ int main(int argc, char *argv[])
         return 0;
     }
 
+    if (strcmp(cmd, "collect_outbox") == 0) { /** **/
+        enqueue_collect_outbox(&snac, url);
+        return 0;
+    }
+
     if (strcmp(cmd, "insert") == 0) { /** **/
         int status;
         xs *data = NULL;
@@ -801,20 +807,40 @@ int main(int argc, char *argv[])
         xs *attl = xs_list_new();
         const char *fn = NULL;
         const char *in_reply_to = NULL;
-        const char **next = NULL;
+        const char **arg_irt = NULL;
+        int arg_date = 0;
+        xs *post_date = NULL;
 
         /* iterate possible attachments */
         while ((fn = GET_ARGV())) {
             FILE *f;
 
-            if (next) {
-                *next = fn;
-                next = NULL;
+            if (arg_irt) {
+                *arg_irt = fn;
+                arg_irt = NULL;
+            }
+            else
+            if (arg_date) {
+                /* convert to ISO */
+                time_t t = xs_parse_localtime(fn, "%Y%m%d%H%M%S");
+
+                if (t == 0) {
+                    fprintf(stderr, "Invalid scheduled date format (must be YYYYmmddHHMMSS)\n");
+                    return 1;
+                }
+
+                post_date = xs_str_iso_date(t);
+                arg_date = 0;
             }
             else
             if (strcmp(fn, "-r") == 0) {
                 /* next argument is an inReplyTo */
-                next = &in_reply_to;
+                arg_irt = &in_reply_to;
+            }
+            else
+            if (strcmp(fn, "-d") == 0) {
+                /* next argument is the schedule date */
+                arg_date = 1;
             }
             else
             if ((f = fopen(fn, "rb")) != NULL) {
@@ -879,7 +905,7 @@ int main(int argc, char *argv[])
             content = xs_dup(url);
 
         if (!content || !*content) {
-            printf("Nothing to send\n");
+            fprintf(stderr, "Nothing to send\n");
             return 1;
         }
 
@@ -890,19 +916,25 @@ int main(int argc, char *argv[])
         if (strcmp(cmd, "note_unlisted") == 0)
             scope = 2;
 
-        msg = msg_note(&snac, content, NULL, in_reply_to, attl, scope, getenv("LANG"), NULL);
+        msg = msg_note(&snac, content, NULL, in_reply_to, attl, scope, getenv("LANG"), post_date);
 
-        c_msg = msg_create(&snac, msg);
+        const char *id = xs_dict_get(msg, "id");
 
-        if (dbglevel) {
-            xs_json_dump(c_msg, 4, stdout);
+        if (post_date)
+            schedule_add(&snac, id, msg);
+        else {
+            c_msg = msg_create(&snac, msg);
+
+            if (dbglevel) {
+                xs_json_dump(c_msg, 4, stdout);
+            }
+
+            enqueue_message(&snac, c_msg);
+            enqueue_webmention(msg);
+
+            timeline_add(&snac, id, msg);
         }
 
-        enqueue_message(&snac, c_msg);
-        enqueue_webmention(msg);
-
-        timeline_add(&snac, xs_dict_get(msg, "id"), msg);
-
         return 0;
     }
 
